@@ -3,12 +3,13 @@ Provides validators for different types of values. Validator validates if the
 value belongs to the given type and is in the provided range.
 """
 import math
-from typing import Union, Optional, Tuple, Any, Hashable
+from typing import Union, Optional, Tuple, Any, Hashable, Generic, TypeVar, cast
 # rename on import since this file implements its own classes
 # with these names.
 from typing import Callable as TCallable
 from typing import Sequence as TSequence
 from typing import List as TList
+from typing import Dict as TDict
 
 import collections
 
@@ -24,7 +25,9 @@ shape_type = Union[int, TCallable[[], int]]
 shape_tuple_type = Optional[Tuple[shape_type, ...]]
 
 
-def validate_all(*args, context: str = '') -> None:
+def validate_all(*args: Tuple["Validator[Any]", Any],
+                 context: str = ''
+                 ) -> None:
     """
     Takes a list of (validator, value) couplets and tests whether they are
     all valid, raising ValueError otherwise.
@@ -60,7 +63,10 @@ def range_str(min_val: Optional[Union[float, np.floating, np.integer]],
         return ''
 
 
-class Validator:
+T = TypeVar("T")
+
+
+class Validator(Generic[T]):
     """
     Base class for all value validators
     each validator should implement:
@@ -94,24 +100,24 @@ class Validator:
     Alternatively you may override ``_valid_values`` and provide your own
     implementation of getting valid values.
     """
-    _valid_values: Tuple = ()
+    _valid_values: Tuple[T, ...] = ()
     is_numeric = False  # is this a numeric type (so it can be swept)?
 
-    def validate(self, value, context: str = ''):
+    def validate(self, value: T, context: str = '') -> None:
         raise NotImplementedError
 
     @property
-    def valid_values(self) -> Tuple:
+    def valid_values(self) -> Tuple[T, ...]:
         return self._valid_values
 
 
-class Anything(Validator):
+class Anything(Validator[Any]):
     """Allow any value to pass."""
 
     def __init__(self) -> None:
         self._valid_values = (0,)
 
-    def validate(self, value: Any, context: str = ''):
+    def validate(self, value: Any, context: str = '') -> None:
         pass
 
     # NOTE(giulioungaretti): why is_numeric?
@@ -123,7 +129,7 @@ class Anything(Validator):
         return '<Anything>'
 
 
-class Nothing(Validator):
+class Nothing(Validator[Any]):
     """
     Allow no value to pass.
     """
@@ -134,14 +140,14 @@ class Nothing(Validator):
         else:
             self.reason = "Nothing Validator"
 
-    def validate(self, value, context=''):
+    def validate(self, value: Any, context: str = '') -> None:
         raise RuntimeError(f"{self.reason}; {context}")
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return f'<Nothing({self.reason})>'
 
 
-class Bool(Validator):
+class Bool(Validator[bool]):
     """
     Requires a boolean.
     """
@@ -168,7 +174,7 @@ class Bool(Validator):
         return '<Boolean>'
 
 
-class Strings(Validator):
+class Strings(Validator[str]):
     """
     Requires a string
     optional parameters min_length and max_length limit the allowed length
@@ -192,7 +198,7 @@ class Strings(Validator):
                             'no smaller than min_length')
         self._valid_values = ('.' * min_length,)
 
-    def validate(self, value: str, context: str = ''):
+    def validate(self, value: str, context: str = '') -> None:
         """
         Validates if string else raises error.
 
@@ -222,7 +228,7 @@ class Strings(Validator):
         return '<Strings{}>'.format(range_str(minv, maxv, 'len'))
 
 
-class Numbers(Validator):
+class Numbers(Validator[numbertypes]):
     """
     Requires a number  of type int, float, numpy.integer or numpy.floating.
 
@@ -284,7 +290,7 @@ class Numbers(Validator):
         return '<Numbers{}>'.format(range_str(minv, maxv, 'v'))
 
 
-class Ints(Validator):
+class Ints(Validator[Union[int, np.integer]]):
     """
     Requires an integer.
     Optional parameters min_value and max_value, enforce
@@ -384,7 +390,7 @@ class PermissiveInts(Ints):
         super().validate(castvalue, context=context)
 
 
-class ComplexNumbers(Validator):
+class ComplexNumbers(Validator[Union[complex, np.complexfloating]]):
     """
     A validator for complex numbers.
     """
@@ -395,7 +401,11 @@ class ComplexNumbers(Validator):
 
         self._valid_values = ((1+1j), )
 
-    def validate(self, value: numbertypes, context: str = '') -> None:
+    def validate(
+            self,
+            value: Union[complex, np.complexfloating],
+            context: str = ''
+    ) -> None:
         """
         Validates if complex number else raises error.
 
@@ -416,7 +426,7 @@ class ComplexNumbers(Validator):
         return '<Complex Number>'
 
 
-class Enum(Validator):
+class Enum(Validator[Hashable]):
     """
     Requires one of a provided set of values.
     eg. Enum(val1, val2, val3)
@@ -432,7 +442,7 @@ class Enum(Validator):
         self._values = set(values)
         self._valid_values = tuple(values)
 
-    def validate(self, value, context: str = '') -> None:
+    def validate(self, value: Hashable, context: str = '') -> None:
         try:
             if value not in self._values:
                 raise ValueError('{} is not in {}; {}'.format(
@@ -447,14 +457,15 @@ class Enum(Validator):
         return '<Enum: {}>'.format(repr(self._values))
 
 
-class OnOff(Validator):
+class OnOff(Validator[str]):
     """
     Requires either the string 'on' or 'off'.
     """
 
     def __init__(self) -> None:
         self._validator = Enum('on', 'off')
-        self._valid_values = self._validator._valid_values
+        self._valid_values = cast(Tuple[str, ...],
+                                  self._validator._valid_values)
 
     def validate(self, value: str, context: str = '') -> None:
         self._validator.validate(value, context)
@@ -475,7 +486,7 @@ class Multiples(Ints):
         min_value: value must be >= min_value
     """
 
-    def __init__(self, divisor: int = 1, **kwargs) -> None:
+    def __init__(self, divisor: int = 1, **kwargs: Any) -> None:
         super().__init__(**kwargs)
         if not isinstance(divisor, int) or divisor <= 0:
             raise TypeError('divisor must be a positive integer')
@@ -507,7 +518,7 @@ class Multiples(Ints):
     is_numeric = True
 
 
-class PermissiveMultiples(Validator):
+class PermissiveMultiples(Validator[numbertypes]):
     """
     A validator that checks whether a value is an integer multiple
     of a fixed divisor (to within some precision). If both value and
@@ -576,7 +587,7 @@ class PermissiveMultiples(Validator):
     is_numeric = True
 
 
-class MultiType(Validator):
+class MultiType(Validator[Any]):
     """
     Allow the union of several different validators.
     For example, to allow numbers as well as "off":
@@ -589,7 +600,7 @@ class MultiType(Validator):
             argument is not a valid validator.
     """
 
-    def __init__(self, *validators: Validator) -> None:
+    def __init__(self, *validators: Validator[Any]) -> None:
         if not validators:
             raise TypeError('MultiType needs at least one Validator')
 
@@ -627,7 +638,7 @@ class MultiType(Validator):
         return '<MultiType: {}>'.format(', '.join(parts))
 
 
-class Arrays(Validator):
+class Arrays(Validator[np.ndarray]):
     """
     Validator for numerical numpy arrays of numeric types (int, float, complex).
     By default it validates int and float arrays.
@@ -732,7 +743,6 @@ class Arrays(Validator):
         if shape is not None:
             self._shape = tuple(shape)
 
-
     @property
     def valid_values(self) -> Tuple[np.ndarray]:
         valid_type = self.valid_types[0]
@@ -822,7 +832,7 @@ class Arrays(Validator):
                                               self.shape_unevaluated)
 
 
-class Lists(Validator):
+class Lists(Validator[TList[Any]]):
     """
     Validator for lists
 
@@ -830,7 +840,7 @@ class Lists(Validator):
         elt_validator: Used to validate the individual elements of the list.
     """
 
-    def __init__(self, elt_validator: Validator = Anything()) -> None:
+    def __init__(self, elt_validator: Validator[Any] = Anything()) -> None:
         self._elt_validator = elt_validator
         self._valid_values = ([vval for vval in elt_validator._valid_values],)
 
@@ -859,7 +869,7 @@ class Lists(Validator):
                 self._elt_validator.validate(elt)
 
 
-class Sequence(Validator):
+class Sequence(Validator[TSequence[Any]]):
     """
     Validator for Sequences.
 
@@ -870,7 +880,7 @@ class Sequence(Validator):
         require_sorted: True or False.
     """
 
-    def __init__(self, elt_validator: Validator = Anything(),
+    def __init__(self, elt_validator: Validator[Any] = Anything(),
                  length: Optional[int] = None,
                  require_sorted: bool = False) -> None:
         self._elt_validator = elt_validator
@@ -885,7 +895,7 @@ class Sequence(Validator):
         msg += self._elt_validator.__repr__() + '>'
         return msg
 
-    def validate(self, value: collections.abc.Sequence,
+    def validate(self, value: TSequence[Any],
                  context: str = '') -> None:
         """
         Validates if sequence else raise typeerror.
@@ -913,7 +923,7 @@ class Sequence(Validator):
                 self._elt_validator.validate(elt)
 
 
-class Callable(Validator):
+class Callable(Validator[TCallable[..., Any]]):
     """
     Validator for callables such as functions.
     """
@@ -921,7 +931,7 @@ class Callable(Validator):
     def __init__(self) -> None:
         self._valid_values = (lambda: 0,)
 
-    def validate(self, value: TCallable, context: str = '') -> None:
+    def validate(self, value: TCallable[..., Any], context: str = '') -> None:
         """
         Validates if callable else raise typeerror.
 
@@ -940,7 +950,7 @@ class Callable(Validator):
         return '<Callable>'
 
 
-class Dict(Validator):
+class Dict(Validator[TDict[Hashable, Any]]):
     """
     Validator for dictionaries.
     """
@@ -958,7 +968,7 @@ class Dict(Validator):
         self.allowed_keys = allowed_keys
         self._valid_values = ({0: 1},)
 
-    def validate(self, value: dict, context: str = '') -> None:
+    def validate(self, value: TDict[Hashable, Any], context: str = '') -> None:
         """
         Validates dictionary keys else raise typeerror.
 
